  #include <Wire.h>

  #include <nRF24L01.h> 
  #include <printf.h>
  #include <RF24.h>
  #include <RF24_config.h>
  #include <SPI.h>


  #include <Servo.h>
  #include <Encoder.h> 
  #include <Adafruit_MotorShield.h>
  #include <Adafruit_PWMServoDriver.h>


RF24 radio(9, 10); // Define the NRF24 module pins (CE, CSN)

const uint64_t pipeAddress = 0xF0F0F0F0E1LL; // Receiver pipe address, should match the transmitter

Adafruit_MotorShield AFMS1 = Adafruit_MotorShield(); // First Motor Shield
Adafruit_MotorShield AFMS2 = Adafruit_MotorShield(); // Second Motor Shield

Adafruit_StepperMotor *stepper = AFMS1.getStepper(200, 1);
Adafruit_DCMotor      *leftMotor = AFMS1.getMotor(3);
Adafruit_DCMotor      *rightMotor = AFMS1.getMotor(4);
Adafruit_DCMotor      *leftMotor1 = AFMS2.getMotor(3);
Adafruit_DCMotor      *rightMotor1 = AFMS2.getMotor(4);

Servo flexServo;

struct SensorData {
  int xAxisValue;
  int yAxisValue;
  int steps;
  int flexSensorValue;
};

SensorData sensorReading;
int lastEncoderPosition = 0;

void setup() {
  Serial.begin(9600);
  radio.begin();
  radio.openReadingPipe(1, pipeAddress);
  radio.setPALevel(RF24_PA_HIGH);
  radio.startListening();

  AFMS1.begin();
  stepper->setSpeed(20); // Adjust the speed as needed based on your motor
  AFMS2.begin();
  flexServo.attach(9); // Servo motor pin
}

void loop() {
  // Receive sensor data from the transmitter
  if (radio.available()) {
    radio.read(&sensorReading, sizeof(SensorData));
    
    // Use sensorReading values to control the motors

    int xAxis = sensorReading.xAxisValue; 
    int yAxis = sensorReading.yAxisValue; 
    int steps = sensorReading.steps;
    int flexSensorValue = sensorReading.flexSensorValue;
    int currentEncoderPosition = steps;

    
    if(yAxis > 100 && xAxis <100 && xAxis > -10)
    {
      leftMotor->setSpeed(abs(yAxis + xAxis)); 
      rightMotor->setSpeed(abs(yAxis + xAxis));
      leftMotor1->setSpeed(abs(yAxis + xAxis)); 
      rightMotor1->setSpeed(abs(yAxis + xAxis));
      leftMotor->run(FORWARD);
      rightMotor->run(FORWARD);
      leftMotor1->run(FORWARD);
      rightMotor1->run(FORWARD);
      
    }
    else if(yAxis < -10 && xAxis <100 && xAxis >-10 )
    {
      leftMotor->setSpeed(abs(yAxis + xAxis)); 
      rightMotor->setSpeed(abs(yAxis + xAxis));
      leftMotor->run(BACKWARD);
      rightMotor->run(BACKWARD);
      leftMotor1->setSpeed(abs(yAxis + xAxis)); 
      rightMotor1->setSpeed(abs(yAxis + xAxis));
      leftMotor1->run(BACKWARD);
      rightMotor1->run(BACKWARD);
      
    }
    else if(xAxis > 100 && yAxis <100 && yAxis > -10 )
    {
      leftMotor->setSpeed(abs(yAxis + xAxis)); 
      rightMotor->setSpeed(abs(yAxis + xAxis));
      leftMotor->run(FORWARD);
      rightMotor->run(BACKWARD);
      leftMotor1->setSpeed(abs(yAxis + xAxis)); 
      rightMotor1->setSpeed(abs(yAxis + xAxis));
      leftMotor1->run(FORWARD);
      rightMotor1->run(BACKWARD);
     
    }
    else if(xAxis < -10 && yAxis <100 && yAxis >-100 )
    {
      leftMotor->setSpeed(abs(yAxis + xAxis)); 
      rightMotor->setSpeed(abs(yAxis + xAxis));
      leftMotor->run(BACKWARD);
      rightMotor->run(FORWARD);
      leftMotor1->setSpeed(abs(yAxis + xAxis)); 
      rightMotor1->setSpeed(abs(yAxis + xAxis));
      leftMotor1->run(BACKWARD);
      rightMotor1->run(FORWARD);
      
      
    }
    else 
    {
      leftMotor->setSpeed(0); 
      rightMotor->setSpeed(0);
      leftMotor1->setSpeed(0); 
      rightMotor1->setSpeed(0);
  
      leftMotor->run(RELEASE);
      rightMotor->run(RELEASE);
      leftMotor1->run(RELEASE);
      rightMotor1->run(RELEASE);
      
    }
    
   
  // Check if the encoder position has changed by 1 value
  if (currentEncoderPosition != lastEncoderPosition) {
    if (currentEncoderPosition - lastEncoderPosition == 1) {
      // If the encoder has been turned by 1 value clockwise, move the stepper motor 4 steps clockwise
      stepper->step(20, FORWARD, SINGLE);  // Adjust the number of steps and mode as needed
    } else if (currentEncoderPosition - lastEncoderPosition == -1) {
      // If the encoder has been turned by 1 value counterclockwise, move the stepper motor 4 steps counterclockwise
      stepper->step(20, BACKWARD, SINGLE);  // Adjust the number of steps and mode as needed
    }
    // Update the last encoder position
    lastEncoderPosition = currentEncoderPosition;
  }

    // Control the servo motor based on flex sensor input
    flexServo.write(flexSensorValue);
   
    // Print received data to Serial monitor
    Serial.print("Received Data: ");
    Serial.print(sensorReading.xAxisValue);
    Serial.print(", ");
    Serial.print(sensorReading.yAxisValue);
    Serial.print(", ");
    Serial.print(sensorReading.steps);
    Serial.print(", ");
    Serial.println(sensorReading.flexSensorValue);
  
  }
}
